%% Subjects

SubjectNames = {'RSpert001', 'RSpert002', 'RSpert003', 'RSpert004', ...
                'RSpert005', 'RSpert006', 'RSpert007', 'RSpert008', ...
                'RSpert009', 'RSpert010', 'RSpert011', 'RSpert012', ...
                'RSpert013', 'RSpert014', 'RSpert015', 'RSpert016', ...
                'RSpert017'};
nSubj = numel(SubjectNames);
BandPass = [8, 13];
Seed = [-39, -27, 55];
corr_crit = .4;

%% Find data

for iSubj = 1:nSubj
    
    % Find data
    EpochCoregFiles{iSubj} = bst_process('CallProcess', 'process_select_files_data', [], [], ...
        'subjectname',   SubjectNames{iSubj}, 'tag', 'Raw');
    indCond = strcmp('spont', {EpochCoregFiles{iSubj}.Condition});
    EpochCoregFiles{iSubj} = EpochCoregFiles{iSubj}(indCond);    
    
end

%% Compute subspace

for iSubj = 1:17
    
    iSubj
    [~, ResultFiles] = find_kernel({EpochCoregFiles{iSubj}.FileName}, 'volume', 'dSPM: MEG(Unconstr)');

    % Process: Compute Signal Subspace: Seed AM
    subspFiles(iSubj) = bst_process('CallProcess', 'process_subspace_seedam', ResultFiles, [], ...
        'timewindow',  [], ...
        'coord',       Seed, ...
        'isestimate',  1, ...
        'orient',      [0, 0, 0], ...
        'bandpass',    BandPass, ...
        'reg',         0, ...
        'epsilon',     0.001, ...
        'split',       1, ...
        'win_length',  1, ...
        'win_overlap', 50, ...
        'wfcn',        'hann', ...
        'isproj',      0, ...
        'isoutf',      0, ...
        'issavecov',   0);    
    
    sMat = in_bst_data(subspFiles(iSubj).FileName);
    iVertex = sMat.Options.iVertex;
    
    % Process: Scan signal subspace (fast)
    scanFile(iSubj) = bst_process('CallProcess', 'process_scan_subspace', subspFiles(iSubj), [], ...
        'rsspace', 3, ...  % SNR values
        'ndim',    [], ...
        'alpha',   [], ...
        'snr',     corr_crit, ...
        'order',   1, ...  % first
        'whiten',  2, ...  % Noise covariance
        'niter',   2, ...
        'iseed', iVertex); % iVertex(iSeed)

    % Process: Project on default anatomy: volume
    scanFilesGroup(iSubj) = bst_process('CallProcess', 'process_project_sources', scanFile(iSubj), [], ...
        'headmodeltype', 'volume');  % MRI volume    
    
end

%% Group statistics

% Process: Subspace scanning group statistics
scanFileAvg = bst_process('CallProcess', 'process_scan_subspace_statistics', scanFilesGroup, [], ...
    'niter', 1,...
    'nperm', 1000,...
    'isinterp', 1);

% Process: Subspace scanning group statistics
scanFileAvg = bst_process('CallProcess', 'process_scan_subspace_statistics', scanFilesGroup, [], ...
    'niter', 2,...
    'nperm', 1000,...
    'isinterp', 1);

% Process: Average: Everything
imageFileAvg = bst_process('CallProcess', 'process_average', imagedFilesGroup, [], ...
    'avgtype',         1, ...  % Everything
    'avg_func',        1, ...  % Arithmetic average:  mean(x)
    'weighted',        0, ...
    'scalenormalized', 0);
